# pylint: disable=unused-import, missing-docstring, invalid-name, reimported, import-error, wrong-import-order, no-name-in-module, shadowed-import
# Functional tests for import aliasing
# 1. useless-import-alias
# 2. consider-using-from-import

import collections as collections  # [useless-import-alias]
from collections import OrderedDict as OrderedDict  # [useless-import-alias]
from collections import OrderedDict as o_dict
import os.path as path  # [consider-using-from-import]
import os.path as p
import foo.bar.foobar as foobar  # [consider-using-from-import]
import os
import os as OS
from sys import version
from . import bar as bar  # [useless-import-alias]
from . import bar as Bar
from . import bar
from ..foo import bar as bar  # [useless-import-alias]
from ..foo.bar import foobar as foobar  # [useless-import-alias]
from ..foo.bar import foobar as anotherfoobar
from . import foo as foo, foo2 as bar2  # [useless-import-alias]
from . import foo as bar, foo2 as foo2  # [useless-import-alias]
from . import foo as bar, foo2 as bar2
from foo.bar import foobar as foobar  # [useless-import-alias]
from foo.bar import foobar as foo
from .foo.bar import f as foobar
from ............a import b  # [relative-beyond-top-level]



def foo():
    """function without params, single line docstring"""
    """ not a docstring"""
    return


def foo2():
    """
        function without params, multiline docstring
    """
    """ not a docstring"""
    return


def fun_with_params_no_docstring(a, b="""
    not a
""" """docstring"""):
    pass

def fun_with_params_no_docstring2(a, b=c[foo():], c=\
    """ not a docstring """):
    pass



from typing import NamedTuple
import typing

# with complex annotations
NT1 = NamedTuple("NT1", [("a", int), ("b", tuple[str, ...])])

# with default values as list
NT2 = NamedTuple(
    "NT2",
    [("a", int), ("b", str), ("c", list[bool])],
    defaults=["foo", [True]],
)

# with namespace
NT3 = typing.NamedTuple("NT3", [("a", int), ("b", str)])

# with too many default values
NT4 = NamedTuple(
    "NT4",
    [("a", int), ("b", str)],
    defaults=[1, "bar", "baz"],
)



def x(y):
    if not y:
        return  # error
    return 1


# without return value
def x(y):
    if not y:
        return
    return


def x(y):
    if y:
        return
    print()


# with return value
def x(y):
    if not y:
        return 1
    return 2


def x(y):
    for i in range(10):
        if i > 100:
            return i
    else:
        return 1


def x(y):
    try:
        return 1
    except:
        return 2


def x(y):
    try:
        return 1
    finally:
        return 2


# inner function
def x(y):
    if not y:
        return 1

    def inner():
        return

    return 2


def x(y):
    if not y:
        return

    def inner():
        return 1



"""Test (edge case): accesses of object named `dict`."""

# OK
dict = {"parameters": {}}
dict["parameters"]["userId"] = "userId"
dict["parameters"]["itemId"] = "itemId"
del dict["parameters"]["itemId"]

# F821 Undefined name `key`
# F821 Undefined name `value`
x: dict["key", "value"]

# OK
x: dict[str, str]

# OK
def unimportant(name):
    pass


def dang(dict, set, list):
    unimportant(name=dict["name"])
    unimportant(name=set["name"])
    unimportant(name=list["name"])



###
# Errors
###

# if/elif/else
def x(y):
    if not y:
        return 1
    # error


def x(y):
    if not y:
        print()  # error
    else:
        return 2


def x(y):
    if not y:
        return 1

    print()  # error


# for
def x(y):
    for i in range(10):
        if i > 10:
            return i
    # error


def x(y):
    for i in range(10):
        if i > 10:
            return i
    else:
        print()  # error


###
# Non-errors
###

# raise as last return
def x(y):
    if not y:
        return 1
    raise Exception


# last line in while loop
def x(y):
    while True:
        if y > 0:
            return 1
        y += 1


# exclude empty functions
def x(y):
    return None


# return inner with statement
def x(y):
    with y:
        return 1


async def function():
    async with thing as foo:
        return foo.bar


# assert as last return
def x(y):
    if not y:
        return 1
    assert False, "Sanity check"


# return value within loop
def bar1(x, y, z):
    for i in x:
        if i > y:
            break
        return z


def bar3(x, y, z):
    for i in x:
        if i > y:
            if z:
                break
        else:
            return z
        return None


def bar1(x, y, z):
    for i in x:
        if i < y:
            continue
        return z


def bar3(x, y, z):
    for i in x:
        if i < y:
            if z:
                continue
        else:
            return z
        return None



zip()
zip(range(3))
zip("a", "b")
zip("a", "b", *zip("c"))
zip(zip("a"), strict=False)
zip(zip("a", strict=True))

zip(range(3), strict=True)
zip("a", "b", strict=False)
zip("a", "b", "c", strict=True)



class SingleLineDocstrings():
    ''' Double quotes single line class docstring '''
    ''' Not a docstring '''

    def foo(self, bar='''not a docstring'''):
        ''' Double quotes single line method docstring'''
        pass

    class Nested(foo()[:]): ''' inline docstring '''; pass



###
# Errors
###
def foo2(x, y, w, z):
    for i in x:
        if i < y:  # [no-else-continue]
            continue
        elif i < w:
            continue
        else:
            a = z


def foo5(x, y, z):
    for i in x:
        if i < y:  # [no-else-continue]
            if y:
                a = 4
            else:
                b = 2
            continue
        elif z:
            c = 2
        else:
            c = 3
        continue


###
# Non-errors
###
def foo1(x, y, z):
    for i in x:
        if i < y:  # [no-else-continue]
            continue
        else:
            a = z


def foo3(x, y, z):
    for i in x:
        if i < y:
            a = 1
            if z:  # [no-else-continue]
                b = 2
                continue
            else:
                c = 3
                continue
        else:
            d = 4
            continue


def foo4(x, y):
    for i in x:
        if i < x:  # [no-else-continue]
            if y:
                a = 4
            else:
                b = 2
            continue
        else:
            c = 3
        continue


def foo6(x, y):
    for i in x:
        if i < x:
            if y:  # [no-else-continue]
                a = 4
                continue
            else:
                b = 2
        else:
            c = 3
        continue


def bar4(x):
    for i in range(10):
        if x:  # [no-else-continue]
            continue
        else:
            try:
                return
            except ValueError:
                continue


def bar1(x, y, z):
    for i in x:
        if i < y:
            continue
        return z


def bar3(x, y, z):
    for i in x:
        if i < y:
            if z:
                continue
        else:
            return z
        return None



s = "qwe"
s.strip(s)  # no warning
s.strip("we")  # no warning
s.strip(".facebook.com")  # warning
s.strip("e")  # no warning
s.strip("\n\t ")  # no warning
s.strip(r"\n\t ")  # warning
s.lstrip(s)  # no warning
s.lstrip("we")  # no warning
s.lstrip(".facebook.com")  # warning
s.lstrip("e")  # no warning
s.lstrip("\n\t ")  # no warning
s.lstrip(r"\n\t ")  # warning
s.rstrip(s)  # no warning
s.rstrip("we")  # warning
s.rstrip(".facebook.com")  # warning
s.rstrip("e")  # no warning
s.rstrip("\n\t ")  # no warning
s.rstrip(r"\n\t ")  # warning

from somewhere import other_type, strip

strip("we")  # no warning
other_type().lstrip()  # no warning
other_type().rstrip(["a", "b", "c"])  # no warning
other_type().strip("a", "b")  # no warning



if x is "abc":
    pass

if 123 is not y:
    pass

if 123 is \
        not y:
    pass

if "123" is x < 3:
    pass

if "123" != x is 3:
    pass

if ("123" != x) is 3:
    pass

if "123" != (x is 3):
    pass



# Ruff has no way of knowing if the following are F505s
a = "wrong"
"%(a)s %(c)s" % {a: "?", "b": "!"}  # F504 ("b" not used)

hidden = {"a": "!"}
"%(a)s %(c)s" % {"x": 1, **hidden}  # Ok (cannot see through splat)



from typing import TypedDict, NotRequired, Literal
import typing

# dict literal
MyType1 = TypedDict("MyType1", {"a": int, "b": str})

# dict call
MyType2 = TypedDict("MyType2", dict(a=int, b=str))

# kwargs
MyType3 = TypedDict("MyType3", a=int, b=str)

# Empty TypedDict
MyType4 = TypedDict("MyType4")

# Literal values
MyType5 = TypedDict("MyType5", {"a": "hello"})
MyType6 = TypedDict("MyType6", a="hello")

# NotRequired
MyType7 = TypedDict("MyType7", {"a": NotRequired[dict]})

# total
MyType8 = TypedDict("MyType8", {"x": int, "y": int}, total=False)

# invalid identifiers
MyType9 = TypedDict("MyType9", {"in": int, "x-y": int})

# using Literal type
MyType10 = TypedDict("MyType10", {"key": Literal["value"]})

# using namespace TypedDict
MyType11 = typing.TypedDict("MyType11", {"key": int})



from __future__ import nested_scopes, generators
from __future__ import with_statement, unicode_literals
from __future__ import absolute_import, division
from __future__ import generator_stop
from __future__ import print_function, generator_stop
from __future__ import invalid_module, generators

if True:
    from __future__ import generator_stop
    from __future__ import generators

if True:
    from __future__ import generator_stop
    from __future__ import invalid_module, generators



"""Test unnecessary direct calls to lambda expressions."""
# pylint: disable=undefined-variable, line-too-long

y = (lambda x: x**2 + 2*x + 1)(a)  # [unnecessary-direct-lambda-call]
y = max((lambda x: x**2)(a), (lambda x: x+1)(a))  # [unnecessary-direct-lambda-call,unnecessary-direct-lambda-call]



def ok(first, default="default"):
    pass


def default(first, password="default"):
    pass


def ok_posonly(first, /, pos, default="posonly"):
    pass


def default_posonly(first, /, pos, password="posonly"):
    pass


def ok_kwonly(first, *, default="kwonly"):
    pass


def default_kwonly(first, *, password="kwonly"):
    pass


def ok_all(first, /, pos, default="posonly", *, kwonly="kwonly"):
    pass


def default_all(first, /, pos, secret="posonly", *, password="kwonly"):
    pass



"""
Should emit:
B025 - on lines 15, 22, 31
"""

import pickle

try:
    a = 1
except ValueError:
    a = 2
finally:
    a = 3

try:
    a = 1
except ValueError:
    a = 2
except ValueError:
    a = 2

try:
    a = 1
except pickle.PickleError:
    a = 2
except ValueError:
    a = 2
except pickle.PickleError:
    a = 2

try:
    a = 1
except (ValueError, TypeError):
    a = 2
except ValueError:
    a = 2
except (OSError, TypeError):
    a = 2



breakpoint()


import pdb
import builtins
from builtins import breakpoint
from pdb import set_trace as st
from celery.contrib.rdb import set_trace
from celery.contrib import rdb
import celery.contrib.rdb


breakpoint()
st()
set_trace()



"""
Should emit:
B017 - on lines 20
"""
import asyncio
import unittest

CONSTANT = True


def something_else() -> None:
    for i in (1, 2, 3):
        print(i)


class Foo:
    pass


class Foobar(unittest.TestCase):
    def evil_raises(self) -> None:
        with self.assertRaises(Exception):
            raise Exception("Evil I say!")

    def context_manager_raises(self) -> None:
        with self.assertRaises(Exception) as ex:
            raise Exception("Context manager is good")
        self.assertEqual("Context manager is good", str(ex.exception))

    def regex_raises(self) -> None:
        with self.assertRaisesRegex(Exception, "Regex is good"):
            raise Exception("Regex is good")

    def raises_with_absolute_reference(self):
        with self.assertRaises(asyncio.CancelledError):
            Foo()



this_should_raise_Q003 = "This is a \"string\""
this_is_fine = "'This' is a \"string\""
this_is_fine = 'This is a "string"'
this_is_fine = '\'This\' is a "string"'
this_is_fine = r"This is a \"string\""



import a
# Don't take this comment into account when determining whether the next import can fit on one line.
from b import c
from d import e  # Do take this comment into account when determining whether the next import can fit on one line.



x = "𝐁ad string"
y = "−"


def f():
    """Here's a docstring with an unusual parenthesis: ）"""
    # And here's a comment with an unusual punctuation mark: ᜵
    ...


def g():
    """Here's a docstring with a greek rho: ρ"""
    # And here's a comment with a greek alpha: ∗
    ...



x = [1, 2, 3]
list(list(x))
list(tuple(x))
tuple(list(x))
tuple(tuple(x))
set(set(x))
set(list(x))
set(tuple(x))
set(sorted(x))
set(reversed(x))
sorted(list(x))
sorted(tuple(x))
sorted(sorted(x))
sorted(reversed(x))



from abc import abstractmethod
from typing_extensions import override


###
# Unused arguments on functions.
###
def f(self, x):
    print("Hello, world!")


def f(cls, x):
    print("Hello, world!")


def f(self, x):
    ...


def f(cls, x):
    ...


###
# Unused arguments on lambdas.
###
lambda x: print("Hello, world!")


class X:
    ###
    # Unused arguments.
    ###
    def f(self, x):
        print("Hello, world!")

    def f(self, /, x):
        print("Hello, world!")

    def f(cls, x):
        print("Hello, world!")

    @classmethod
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    def f(x):
        print("Hello, world!")

    ###
    # Unused arguments attached to empty functions (OK).
    ###
    def f(self, x):
        ...

    def f(self, /, x):
        ...

    def f(cls, x):
        ...

    @classmethod
    def f(cls, x):
        ...

    @staticmethod
    def f(cls, x):
        ...

    @staticmethod
    def f(x):
        ...

    ###
    # Unused functions attached to abstract methods (OK).
    ###
    @abstractmethod
    def f(self, x):
        print("Hello, world!")

    @abstractmethod
    def f(self, /, x):
        print("Hello, world!")

    @abstractmethod
    def f(cls, x):
        print("Hello, world!")

    @classmethod
    @abstractmethod
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    @abstractmethod
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    @abstractmethod
    def f(x):
        print("Hello, world!")

    ###
    # Unused functions attached to overrides (OK).
    ###
    @override
    def f(self, x):
        print("Hello, world!")

    @override
    def f(self, /, x):
        print("Hello, world!")

    @override
    def f(cls, x):
        print("Hello, world!")

    @classmethod
    @override
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    @override
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    @override
    def f(x):
        print("Hello, world!")



from __future__ import all_feature_names
import functools, os
from datetime import datetime
from collections import (
    Counter,
    OrderedDict,
    namedtuple,
)
import multiprocessing.pool
import multiprocessing.process
import logging.config
import logging.handlers
from typing import (
    TYPE_CHECKING,
    NamedTuple,
    Dict,
    Type,
    TypeVar,
    List,
    Set,
    Union,
    cast,
)
from dataclasses import MISSING, field

from blah import ClassA, ClassB, ClassC

if TYPE_CHECKING:
    from models import Fruit, Nut, Vegetable

if TYPE_CHECKING:
    import shelve
    import importlib

if TYPE_CHECKING:
    """Hello, world!"""
    import pathlib

    z = 1


class X:
    datetime: datetime
    foo: Type["NamedTuple"]

    def a(self) -> "namedtuple":
        x = os.environ["1"]
        y = Counter()
        z = multiprocessing.pool.ThreadPool()

    def b(self) -> None:
        import pickle


__all__ = ["ClassA"] + ["ClassB"]
__all__ += ["ClassC"]

X = TypeVar("X")
Y = TypeVar("Y", bound="Dict")
Z = TypeVar("Z", "List", "Set")

a = list["Fruit"]
b = Union["""Nut""", None]
c = cast("Vegetable", b)

Field = lambda default=MISSING: field(default=default)


# Test: access a sub-importation via an alias.
import pyarrow as pa
import pyarrow.csv

print(pa.csv.read_csv("test.csv"))


# Test: referencing an import via TypeAlias.
import sys

import numpy as np

if sys.version_info >= (3, 10):
    from typing import TypeAlias
else:
    from typing_extensions import TypeAlias


CustomInt: TypeAlias = "np.int8 | np.int16"



import functools
from functools import lru_cache


@lru_cache()
def fixme1():
    pass


@other_deco_after
@functools.lru_cache()
def fixme2():
    pass


@lru_cache(maxsize=None)
def fixme3():
    pass


@functools.lru_cache(maxsize=None)
@other_deco_before
def fixme4():
    pass


@lru_cache( # A 
) # B
def fixme5():
    pass


@lru_cache(
    # A
)   # B
def fixme6():
    pass


@functools.lru_cache(
    # A
    maxsize = None) # B
def fixme7():
    pass


@functools.lru_cache(
    # A1
    maxsize = None
    # A2
) # B
def fixme8():
    pass


@functools.lru_cache(
    # A1
    maxsize = 
    None
    # A2

)
def fixme9():
    pass


@functools.lru_cache(
    # A1
    maxsize = 
    None
    # A2
)
def fixme10():
    pass


@lru_cache
def correct1():
    pass


@functools.lru_cache
def correct2():
    pass


@functoools.lru_cache(maxsize=64)
def correct3():
    pass


def user_func():
    pass


@lru_cache(user_func)
def correct4():
    pass


@lru_cache(user_func, maxsize=None)
def correct5():
    pass



"""Test: inner class annotation."""


class RandomClass:
    def random_func(self) -> "InnerClass":
        pass


class OuterClass:
    class InnerClass:
        pass

    def failing_func(self) -> "InnerClass":
        return self.InnerClass()



#: E712
if res == True:
    pass
#: E712
if res != False:
    pass
#: E712
if True != res:
    pass
#: E712
if False == res:
    pass
#: E712
if res[1] == True:
    pass
#: E712
if res[1] != False:
    pass
#: E712
var = 1 if cond == True else -1 if cond == False else cond
#: E712
if (True) == TrueElement or x == TrueElement:
    pass

if res == True != False:
    pass

#: Okay
if x not in y:
    pass

if not (X in Y or X is Z):
    pass

if not (X in Y):
    pass

if x is not y:
    pass

if X is not Y is not Z:
    pass

if TrueElement.get_element(True) == TrueElement.get_element(False):
    pass

assert (not foo) in bar
assert {"x": not foo} in bar
assert [42, not foo] in bar



"""Test: typing module imports."""
from foo import cast

# OK
x = cast("Model")

from typing import cast


# F821 Undefined name `Model`
x = cast("Model", x)


import typing


# F821 Undefined name `Model`
x = typing.cast("Model", x)


from typing import Pattern

# F821 Undefined name `Model`
x = Pattern["Model"]


from typing.re import Match

# F821 Undefined name `Model`
x = Match["Model"]



# No docstring, so we can test D100
from functools import wraps
import os
from .expected import Expectation
from typing import overload
from typing_extensions import override


expectation = Expectation()
expect = expectation.expect

expect('class_', 'D101: Missing docstring in public class')


class class_:

    expect('meta', 'D419: Docstring is empty')

    class meta:
        """"""

    @expect('D102: Missing docstring in public method')
    def method(self=None):
        pass

    def _ok_since_private(self=None):
        pass

    @overload
    def overloaded_method(self, a: int) -> str:
        ...

    @overload
    def overloaded_method(self, a: str) -> str:
        """Foo bar documentation."""
        ...

    def overloaded_method(a):
        """Foo bar documentation."""
        return str(a)

    expect('overloaded_method',
           "D418: Function/ Method decorated with @overload"
           " shouldn't contain a docstring")

    @override
    def overridden_method(a):
        return str(a)

    @property
    def foo(self):
        """The foo of the thing, which isn't in imperative mood."""
        return "hello"

    @expect('D102: Missing docstring in public method')
    def __new__(self=None):
        pass

    @expect('D107: Missing docstring in __init__')
    def __init__(self=None):
        pass

    @expect('D105: Missing docstring in magic method')
    def __str__(self=None):
        pass

    @expect('D102: Missing docstring in public method')
    def __call__(self=None, x=None, y=None, z=None):
        pass


@expect('D419: Docstring is empty')
def function():
    """ """
    def ok_since_nested():
        pass

    @expect('D419: Docstring is empty')
    def nested():
        ''


def function_with_nesting():
    """Foo bar documentation."""
    @overload
    def nested_overloaded_func(a: int) -> str:
        ...

    @overload
    def nested_overloaded_func(a: str) -> str:
        """Foo bar documentation."""
        ...

    def nested_overloaded_func(a):
        """Foo bar documentation."""
        return str(a)


expect('nested_overloaded_func',
       "D418: Function/ Method decorated with @overload"
       " shouldn't contain a docstring")


@overload
def overloaded_func(a: int) -> str:
    ...


@overload
def overloaded_func(a: str) -> str:
    """Foo bar documentation."""
    ...


def overloaded_func(a):
    """Foo bar documentation."""
    return str(a)


expect('overloaded_func',
       "D418: Function/ Method decorated with @overload"
       " shouldn't contain a docstring")


@expect('D200: One-line docstring should fit on one line with quotes '
        '(found 3)')
@expect('D212: Multi-line docstring summary should start at the first line')
def asdlkfasd():
    """
    Wrong.
    """


@expect('D201: No blank lines allowed before function docstring (found 1)')
def leading_space():

    """Leading space."""


@expect('D202: No blank lines allowed after function docstring (found 1)')
def trailing_space():
    """Leading space."""

    pass


@expect('D201: No blank lines allowed before function docstring (found 1)')
@expect('D202: No blank lines allowed after function docstring (found 1)')
def trailing_and_leading_space():

    """Trailing and leading space."""

    pass


expect('LeadingSpaceMissing',
       'D203: 1 blank line required before class docstring (found 0)')


class LeadingSpaceMissing:
    """Leading space missing."""


expect('WithLeadingSpace',
       'D211: No blank lines allowed before class docstring (found 1)')


class WithLeadingSpace:

    """With leading space."""


expect('TrailingSpace',
       'D204: 1 blank line required after class docstring (found 0)')
expect('TrailingSpace',
       'D211: No blank lines allowed before class docstring (found 1)')


class TrailingSpace:

    """TrailingSpace."""
    pass


expect('LeadingAndTrailingSpaceMissing',
       'D203: 1 blank line required before class docstring (found 0)')
expect('LeadingAndTrailingSpaceMissing',
       'D204: 1 blank line required after class docstring (found 0)')


class LeadingAndTrailingSpaceMissing:
    """Leading and trailing space missing."""
    pass


@expect('D205: 1 blank line required between summary line and description '
        '(found 0)')
@expect('D213: Multi-line docstring summary should start at the second line')
def multi_line_zero_separating_blanks():
    """Summary.
    Description.

    """


@expect('D205: 1 blank line required between summary line and description '
        '(found 2)')
@expect('D213: Multi-line docstring summary should start at the second line')
def multi_line_two_separating_blanks():
    """Summary.


    Description.

    """


@expect('D213: Multi-line docstring summary should start at the second line')
def multi_line_one_separating_blanks():
    """Summary.

    Description.

    """


@expect('D207: Docstring is under-indented')
@expect('D213: Multi-line docstring summary should start at the second line')
def asdfsdf():
    """Summary.

Description.

    """


@expect('D207: Docstring is under-indented')
@expect('D213: Multi-line docstring summary should start at the second line')
def asdsdfsdffsdf():
    """Summary.

    Description.

"""


@expect('D208: Docstring is over-indented')
@expect('D213: Multi-line docstring summary should start at the second line')
def asdfsdsdf24():
    """Summary.

       Description.

    """


@expect('D208: Docstring is over-indented')
@expect('D213: Multi-line docstring summary should start at the second line')
def asdfsdsdfsdf24():
    """Summary.

    Description.

        """


@expect('D208: Docstring is over-indented')
@expect('D213: Multi-line docstring summary should start at the second line')
def asdfsdfsdsdsdfsdf24():
    """Summary.

        Description.

    """


@expect('D209: Multi-line docstring closing quotes should be on a separate '
        'line')
@expect('D213: Multi-line docstring summary should start at the second line')
def asdfljdf24():
    """Summary.

    Description."""


@expect('D210: No whitespaces allowed surrounding docstring text')
def endswith():
    """Whitespace at the end. """


@expect('D210: No whitespaces allowed surrounding docstring text')
def around():
    """ Whitespace at everywhere. """


@expect('D210: No whitespaces allowed surrounding docstring text')
@expect('D213: Multi-line docstring summary should start at the second line')
def multiline():
    """ Whitespace at the beginning.

    This is the end.
    """


@expect('D300: Use """triple double quotes""" (found \'\'\'-quotes)')
def triple_single_quotes_raw():
    r'''Summary.'''


@expect('D300: Use """triple double quotes""" (found \'\'\'-quotes)')
def triple_single_quotes_raw_uppercase():
    R'''Summary.'''


@expect('D300: Use """triple double quotes""" (found \'-quotes)')
def single_quotes_raw():
    r'Summary.'


@expect('D300: Use """triple double quotes""" (found \'-quotes)')
def single_quotes_raw_uppercase():
    R'Summary.'


@expect('D300: Use """triple double quotes""" (found \'-quotes)')
@expect('D301: Use r""" if any backslashes in a docstring')
def single_quotes_raw_uppercase_backslash():
    R'Sum\mary.'


@expect('D301: Use r""" if any backslashes in a docstring')
def double_quotes_backslash():
    """Sum\\mary."""


@expect('D301: Use r""" if any backslashes in a docstring')
def double_quotes_backslash_uppercase():
    R"""Sum\\mary."""


@expect('D213: Multi-line docstring summary should start at the second line')
def exceptions_of_D301():
    """Exclude some backslashes from D301.

    In particular, line continuations \
    and unicode literals \u0394 and \N{GREEK CAPITAL LETTER DELTA}.
    They are considered to be intentionally unescaped.
    """


@expect("D400: First line should end with a period (not 'y')")
@expect("D415: First line should end with a period, question mark, "
        "or exclamation point (not 'y')")
def lwnlkjl():
    """Summary"""


@expect("D401: First line should be in imperative mood "
        "(perhaps 'Return', not 'Returns')")
def liouiwnlkjl():
    """Returns foo."""


@expect("D401: First line should be in imperative mood; try rephrasing "
        "(found 'Constructor')")
def sdgfsdg23245():
    """Constructor for a foo."""


@expect("D401: First line should be in imperative mood; try rephrasing "
        "(found 'Constructor')")
def sdgfsdg23245777():
    """Constructor."""


@expect('D402: First line should not be the function\'s "signature"')
def foobar():
    """Signature: foobar()."""


@expect('D213: Multi-line docstring summary should start at the second line')
def new_209():
    """First line.

    More lines.
    """
    pass


@expect('D213: Multi-line docstring summary should start at the second line')
def old_209():
    """One liner.

    Multi-line comments. OK to have extra blank line

    """


@expect("D103: Missing docstring in public function")
def oneliner_d102(): return


@expect("D400: First line should end with a period (not 'r')")
@expect("D415: First line should end with a period, question mark,"
        " or exclamation point (not 'r')")
def oneliner_withdoc(): """One liner"""


def ignored_decorator(func):   # noqa: D400,D401,D415
    """Runs something"""
    func()
    pass


def decorator_for_test(func):   # noqa: D400,D401,D415
    """Runs something"""
    func()
    pass


@ignored_decorator
def oneliner_ignored_decorator(): """One liner"""


@decorator_for_test
@expect("D400: First line should end with a period (not 'r')")
@expect("D415: First line should end with a period, question mark,"
        " or exclamation point (not 'r')")
def oneliner_with_decorator_expecting_errors(): """One liner"""


@decorator_for_test
def valid_oneliner_with_decorator(): """One liner."""


@expect("D207: Docstring is under-indented")
@expect('D213: Multi-line docstring summary should start at the second line')
def docstring_start_in_same_line(): """First Line.

    Second Line
    """


def function_with_lambda_arg(x=lambda y: y):
    """Wrap the given lambda."""


@expect('D213: Multi-line docstring summary should start at the second line')
def a_following_valid_function(x=None):
    """Check for a bug where the previous function caused an assertion.

    The assertion was caused in the next function, so this one is necessary.

    """


def outer_function():
    """Do something."""
    def inner_function():
        """Do inner something."""
        return 0


@expect("D400: First line should end with a period (not 'g')")
@expect("D401: First line should be in imperative mood "
        "(perhaps 'Run', not 'Runs')")
@expect("D415: First line should end with a period, question mark, "
        "or exclamation point (not 'g')")
def docstring_bad():
    """Runs something"""
    pass


def docstring_bad_ignore_all():  # noqa
    """Runs something"""
    pass


def docstring_bad_ignore_one():  # noqa: D400,D401,D415
    """Runs something"""
    pass


@expect("D401: First line should be in imperative mood "
        "(perhaps 'Run', not 'Runs')")
def docstring_ignore_some_violations_but_catch_D401():  # noqa: E501,D400,D415
    """Runs something"""
    pass


@expect(
    "D401: First line should be in imperative mood "
    "(perhaps 'Initiate', not 'Initiates')"
)
def docstring_initiates():
    """Initiates the process."""


@expect(
    "D401: First line should be in imperative mood "
    "(perhaps 'Initialize', not 'Initializes')"
)
def docstring_initializes():
    """Initializes the process."""


@wraps(docstring_bad_ignore_one)
def bad_decorated_function():
    """Bad (E501) but decorated"""
    pass


def valid_google_string():  # noqa: D400
    """Test a valid something!"""


@expect("D415: First line should end with a period, question mark, "
        "or exclamation point (not 'g')")
def bad_google_string():  # noqa: D400
    """Test a valid something"""


# This is reproducing a bug where AttributeError is raised when parsing class
# parameters as functions for Google / Numpy conventions.
class Blah:  # noqa: D203,D213
    """A Blah.

    Parameters
    ----------
    x : int

    """

    def __init__(self, x):
        pass


expect(os.path.normcase(__file__ if __file__[-1] != 'c' else __file__[:-1]),
       'D100: Missing docstring in public module')


@expect('D201: No blank lines allowed before function docstring (found 1)')
@expect('D213: Multi-line docstring summary should start at the second line')
def multiline_leading_space():

    """Leading space.

    More content.
    """


@expect('D202: No blank lines allowed after function docstring (found 1)')
@expect('D213: Multi-line docstring summary should start at the second line')
def multiline_trailing_space():
    """Leading space.

    More content.
    """

    pass


@expect('D201: No blank lines allowed before function docstring (found 1)')
@expect('D202: No blank lines allowed after function docstring (found 1)')
@expect('D213: Multi-line docstring summary should start at the second line')
def multiline_trailing_and_leading_space():

    """Trailing and leading space.

    More content.
    """

    pass


@expect('D210: No whitespaces allowed surrounding docstring text')
@expect("D400: First line should end with a period (not '\"')")
@expect("D415: First line should end with a period, question mark, "
        "or exclamation point (not '\"')")
def endswith_quote():
    """Whitespace at the end, but also a quote" """



"""
Should emit:
B023 - on lines 12, 13, 16, 28, 29, 30, 31, 40, 42, 50, 51, 52, 53, 61, 68.
"""

functions = []
z = 0

for x in range(3):
    y = x + 1
    # Subject to late-binding problems
    functions.append(lambda: x)
    functions.append(lambda: y)  # not just the loop var

    def f_bad_1():
        return x

    # Actually OK
    functions.append(lambda x: x * 2)
    functions.append(lambda x=x: x)
    functions.append(lambda: z)  # OK because not assigned in the loop

    def f_ok_1(x):
        return x * 2


def check_inside_functions_too():
    ls = [lambda: x for x in range(2)]
    st = {lambda: x for x in range(2)}
    gn = (lambda: x for x in range(2))
    dt = {x: lambda: x for x in range(2)}


async def pointless_async_iterable():
    yield 1


async def container_for_problems():
    async for x in pointless_async_iterable():
        functions.append(lambda: x)

    [lambda: x async for x in pointless_async_iterable()]


a = 10
b = 0
while True:
    a = a_ = a - 1
    b += 1
    functions.append(lambda: a)
    functions.append(lambda: a_)
    functions.append(lambda: b)
    functions.append(lambda: c)  # not a name error because of late binding!
    c: bool = a > 3
    if not c:
        break

# Nested loops should not duplicate reports
for j in range(2):
    for k in range(3):
        lambda: j * k


for j, k, l in [(1, 2, 3)]:

    def f():
        j = None  # OK because it's an assignment
        [l for k in range(2)]  # error for l, not for k

        assert a and functions

    a.attribute = 1  # modifying an attribute doesn't make it a loop variable
    functions[0] = lambda: None  # same for an element

for var in range(2):

    def explicit_capture(captured=var):
        return captured


for i in range(3):
    lambda: f"{i}"



"{".format(1)
"}".format(1)
"{foo[}".format(foo=1)
# too much string recursion (placeholder-in-placeholder)
"{:{:{}}}".format(1, 2, 3)
# ruff picks these issues up, but flake8 doesn't
"{foo[]}".format(foo={"": 1})
"{foo..}".format(foo=1)
"{foo..bar}".format(foo=1)

# The following are all "good" uses of .format
"{.__class__}".format("")
"{foo[bar]}".format(foo={"bar": "barv"})
"{[bar]}".format({"bar": "barv"})
"{:{}} {}".format(1, 15, 2)
"{:2}".format(1)
"{foo}-{}".format(1, foo=2)
a = ()
"{}".format(*a)
k = {}
"{foo}".format(**k)



from typing import overload


@overload
def foo(i: int) -> "int":
    ...


@overload
def foo(i: "str") -> "str":
    ...


def foo(i):
    return i


@overload
def bar(i: int) -> "int":
    ...


@overload
def bar(i: "str") -> "str":
    ...


class X:
    def bar(i):
        return i


# TODO(charlie): This third case should raise an error (as in Mypy), because we have a
# statement between the interfaces and implementation.
@overload
def baz(i: int) -> "int":
    ...


@overload
def baz(i: "str") -> "str":
    ...


x = 1


def baz(i):
    return i



from line_with_88 import aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
from line_with_89 import (
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa,
)

if indented:
    from line_with_88 import aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    from line_with_89 import aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    from line_with_90 import aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    from line_with_91 import aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    from line_with_92 import aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    from line_with_93 import (
        aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa,
    )



# Comment 1
# Comment 2
import D

# Comment 3a
import C

# Comment 3b
import C

import B  # Comment 4

# Comment 5

# Comment 6
from A import (
    a,  # Comment 7
    b,
    c,  # Comment 8
)
from A import (
    a,  # Comment 9
    b,  # Comment 10
    c,  # Comment 11
)



import os
# This is a comment in the same section, so we need to add one newline.
import sys
import numpy as np
# This is a comment, but it starts a new section, so we don't need to add a newline
# before it.
import leading_prefix



def __bad__():
    pass


def __good():
    pass


def good__():
    pass


def nested():
    def __bad__():
        pass

    def __good():
        pass

    def good__():
        pass


class Class:
    def __good__(self):
        pass


# https://peps.python.org/pep-0562/
def __getattr__(name):
    pass


# https://peps.python.org/pep-0562/
def __dir__():
    pass



class Parent:
    def method(self):
        pass

    def wrong(self):
        pass


class Child(Parent):
    def method(self):
        parent = super()  # ok
        super().method()  # ok
        Parent.method(self)  # ok
        Parent.super(1, 2)  # ok

    def wrong(self):
        parent = super(Child, self)  # wrong
        super(Child, self).method  # wrong
        super(
            Child,
            self,
        ).method()  # wrong


class BaseClass:
    def f(self):
        print("f")


def defined_outside(self):
    super(MyClass, self).f()  # CANNOT use super()


class MyClass(BaseClass):
    def normal(self):
        super(MyClass, self).f()  # can use super()
        super().f()

    def different_argument(self, other):
        super(MyClass, other).f()  # CANNOT use super()

    def comprehension_scope(self):
        [super(MyClass, self).f() for x in [1]]  # CANNOT use super()

    def inner_functions(self):
        def outer_argument():
            super(MyClass, self).f()  # CANNOT use super()

        def inner_argument(self):
            super(MyClass, self).f()  # can use super()
            super().f()

        outer_argument()
        inner_argument(self)

    def inner_class(self):
        class InnerClass:
            super(MyClass, self).f()  # CANNOT use super()

            def method(inner_self):
                super(MyClass, self).f()  # CANNOT use super()

        InnerClass().method()

    defined_outside = defined_outside



try:
    1 / 0
except ValueError as e:
    pass


try:
    1 / 0
except ValueError as e:
    print(e)


def f1():
    x = 1
    y = 2
    z = x + y


def f2():
    foo = (1, 2)
    (a, b) = (1, 2)

    bar = (1, 2)
    (c, d) = bar

    (x, y) = baz = bar


def f3():
    locals()
    x = 1


def f4():
    _ = 1
    __ = 1
    _discarded = 1


a = 1


def f5():
    global a

    # Used in `f7` via `nonlocal`.
    b = 1

    def f6():
        # F841
        b = 1

    def f7():
        nonlocal b


def f6():
    annotations = []
    assert len([annotations for annotations in annotations])


def f7():
    def connect():
        return None, None

    with connect() as (connection, cursor):
        cursor.execute("SELECT * FROM users")


def f8():
    with open("file") as f, open("") as ((a, b)):
        print("hello")



# pylint: disable=missing-docstring, too-few-public-methods
from abc import ABCMeta, abstractmethod


class Cls:
    @property
    def attribute(self, param, param1):  # [property-with-parameters]
        return param + param1

    @property
    def attribute_keyword_only(self, *, param, param1):  # [property-with-parameters]
        return param + param1

    @property
    def attribute_positional_only(self, param, param1, /):  # [property-with-parameters]
        return param + param1


class MyClassBase(metaclass=ABCMeta):
    """MyClassBase."""

    @property
    @abstractmethod
    def example(self):
        """Getter."""

    @example.setter
    @abstractmethod
    def example(self, value):
        """Setter."""



"""This is the docstring for the example.py module.  Modules names should
have short, all-lowercase names.  The module name may have underscores if
this improves readability.

Every module should have a docstring at the very top of the file.  The
module's docstring may extend over multiple lines.  If your docstring does
extend over multiple lines, the closing three quotation marks must be on
a line by itself, preferably preceded by a blank line.

"""

# Example source file from the official "numpydoc docstring guide"
# documentation (with the modification of commenting out all the original
# ``import`` lines, plus adding this note and ``Expectation`` code):
#     * As HTML: https://numpydoc.readthedocs.io/en/latest/example.html
#     * Source Python:
#         https://github.com/numpy/numpydoc/blob/master/doc/example.py

# from __future__ import division, absolute_import, print_function
#
# import os  # standard library imports first
#
# Do NOT import using *, e.g. from numpy import *
#
# Import the module using
#
#   import numpy
#
# instead or import individual functions as needed, e.g
#
#  from numpy import array, zeros
#
# If you prefer the use of abbreviated module names, we suggest the
# convention used by NumPy itself::
#
# import numpy as np
# import matplotlib as mpl
# import matplotlib.pyplot as plt
#
# These abbreviated names are not to be used in docstrings; users must
# be able to paste and execute docstrings after importing only the
# numpy module itself, unabbreviated.

import os
from .expected import Expectation

expectation = Expectation()
expect = expectation.expect

# module docstring expected violations:
expectation.expected.add((
    os.path.normcase(__file__),
    "D205: 1 blank line required between summary line and description "
    "(found 0)"))
expectation.expected.add((
    os.path.normcase(__file__),
    "D213: Multi-line docstring summary should start at the second line"))
expectation.expected.add((
    os.path.normcase(__file__),
    "D400: First line should end with a period (not 'd')"))
expectation.expected.add((
    os.path.normcase(__file__),
    "D404: First word of the docstring should not be `This`"))
expectation.expected.add((
    os.path.normcase(__file__),
    "D415: First line should end with a period, question mark, or exclamation "
    "point (not 'd')"))


@expect("D213: Multi-line docstring summary should start at the second line",
        arg_count=3)
@expect("D401: First line should be in imperative mood; try rephrasing "
        "(found 'A')", arg_count=3)
@expect("D413: Missing blank line after last section ('Examples')",
        arg_count=3)
def foo(var1, var2, long_var_name='hi', **kwargs):
    r"""A one-line summary that does not use variable names.

    Several sentences providing an extended description. Refer to
    variables using back-ticks, e.g. `var`.

    Parameters
    ----------
    var1 : array_like
        Array_like means all those objects -- lists, nested lists, etc. --
        that can be converted to an array.  We can also refer to
        variables like `var1`.
    var2 : int
        The type above can either refer to an actual Python type
        (e.g. ``int``), or describe the type of the variable in more
        detail, e.g. ``(N,) ndarray`` or ``array_like``.
    long_var_name : {'hi', 'ho'}, optional
        Choices in brackets, default first when optional.
    **kwargs : int
        More keyword arguments.

    Returns
    -------
    type
        Explanation of anonymous return value of type ``type``.
    describe : type
        Explanation of return value named `describe`.
    out : type
        Explanation of `out`.
    type_without_description

    Other Parameters
    ----------------
    only_seldom_used_keywords : type
        Explanation
    common_parameters_listed_above : type
        Explanation

    Raises
    ------
    BadException
        Because you shouldn't have done that.

    See Also
    --------
    numpy.array : Relationship (optional).
    numpy.ndarray : Relationship (optional), which could be fairly long, in
                    which case the line wraps here.
    numpy.dot, numpy.linalg.norm, numpy.eye

    Notes
    -----
    Notes about the implementation algorithm (if needed).

    This can have multiple paragraphs.

    You may include some math:

    .. math:: X(e^{j\omega } ) = x(n)e^{ - j\omega n}

    And even use a Greek symbol like :math:`\omega` inline.

    References
    ----------
    Cite the relevant literature, e.g. [1]_.  You may also cite these
    references in the notes section above.

    .. [1] O. McNoleg, "The integration of GIS, remote sensing,
       expert systems and adaptive co-kriging for environmental habitat
       modelling of the Highland Haggis using object-oriented, fuzzy-logic
       and neural-network techniques," Computers & Geosciences, vol. 22,
       pp. 585-588, 1996.

    Examples
    --------
    These are written in doctest format, and should illustrate how to
    use the function.

    >>> a = [1, 2, 3]
    >>> print([x + 3 for x in a])
    [4, 5, 6]
    >>> print("a\nb")
    a
    b
    """
    # After closing class docstring, there should be one blank line to
    # separate following codes (according to PEP257).
    # But for function, method and module, there should be no blank lines
    # after closing the docstring.
    pass



"""A one line summary of the module or program, terminated by a period.

Leave one blank line.  The rest of this docstring should contain an
overall description of the module or program.  Optionally, it may also
contain a brief description of exported classes and functions and/or usage
examples.

  Typical usage example:

  foo = ClassFoo()
  bar = foo.FunctionBar()
"""
# above: "2.8.2 Modules" section example
# https://google.github.io/styleguide/pyguide.html#382-modules

# Examples from the official "Google Python Style Guide" documentation:
#     * As HTML: https://google.github.io/styleguide/pyguide.html
#     * Source Markdown:
#         https://github.com/google/styleguide/blob/gh-pages/pyguide.md

import os
from .expected import Expectation

expectation = Expectation()
expect = expectation.expect

# module docstring expected violations:
expectation.expected.add((
    os.path.normcase(__file__),
    "D213: Multi-line docstring summary should start at the second line"))


# "3.8.3 Functions and Methods" section example
# https://google.github.io/styleguide/pyguide.html#383-functions-and-methods
@expect("D213: Multi-line docstring summary should start at the second line",
        arg_count=3)
@expect("D401: First line should be in imperative mood "
        "(perhaps 'Fetch', not 'Fetches')", arg_count=3)
@expect("D406: Section name should end with a newline "
        "('Raises', not 'Raises:')", arg_count=3)
@expect("D406: Section name should end with a newline "
        "('Returns', not 'Returns:')", arg_count=3)
@expect("D407: Missing dashed underline after section ('Raises')", arg_count=3)
@expect("D407: Missing dashed underline after section ('Returns')",
        arg_count=3)
@expect("D413: Missing blank line after last section ('Raises')", arg_count=3)
def fetch_bigtable_rows(big_table, keys, other_silly_variable=None, **kwargs):
    """Fetches rows from a Bigtable.

    Retrieves rows pertaining to the given keys from the Table instance
    represented by big_table.  Silly things may happen if
    other_silly_variable is not None.

    Args:
        big_table: An open Bigtable Table instance.
        keys: A sequence of strings representing the key of each table row
            to fetch.
        other_silly_variable: Another optional variable, that has a much
            longer name than the other args, and which does nothing.
        **kwargs: More keyword arguments.

    Returns:
        A dict mapping keys to the corresponding table row data
        fetched. Each row is represented as a tuple of strings. For
        example:

        {'Serak': ('Rigel VII', 'Preparer'),
         'Zim': ('Irk', 'Invader'),
         'Lrrr': ('Omicron Persei 8', 'Emperor')}

        If a key from the keys argument is missing from the dictionary,
        then that row was not found in the table.

    Raises:
        IOError: An error occurred accessing the bigtable.Table object.
    """


# "3.8.4 Classes" section example
# https://google.github.io/styleguide/pyguide.html#384-classes
@expect("D203: 1 blank line required before class docstring (found 0)")
@expect("D213: Multi-line docstring summary should start at the second line")
@expect("D406: Section name should end with a newline "
        "('Attributes', not 'Attributes:')")
@expect("D407: Missing dashed underline after section ('Attributes')")
@expect("D413: Missing blank line after last section ('Attributes')")
class SampleClass:
    """Summary of class here.

    Longer class information....
    Longer class information....

    Attributes:
        likes_spam: A boolean indicating if we like SPAM or not.
        eggs: An integer count of the eggs we have laid.
    """

    @expect("D401: First line should be in imperative mood "
            "(perhaps 'Init', not 'Inits')", arg_count=2)
    def __init__(self, likes_spam=False):
        """Inits SampleClass with blah."""
        if self:  # added to avoid NameError when run via @expect decorator
            self.likes_spam = likes_spam
            self.eggs = 0

    @expect("D401: First line should be in imperative mood "
            "(perhaps 'Perform', not 'Performs')", arg_count=1)
    def public_method(self):
        """Performs operation blah."""



open("foo", "U")
open("foo", "Ur")
open("foo", "Ub")
open("foo", "rUb")
open("foo", "r")
open("foo", "rt")
open("f", "r", encoding="UTF-8")
open("f", "wt")

with open("foo", "U") as f:
    pass
with open("foo", "Ur") as f:
    pass
with open("foo", "Ub") as f:
    pass
with open("foo", "rUb") as f:
    pass
with open("foo", "r") as f:
    pass
with open("foo", "rt") as f:
    pass
with open("foo", "r", encoding="UTF-8") as f:
    pass
with open("foo", "wt") as f:
    pass

open(f("a", "b", "c"), "U")
open(f("a", "b", "c"), "Ub")

with open(f("a", "b", "c"), "U") as f:
    pass
with open(f("a", "b", "c"), "Ub") as f:
    pass

with open("foo", "U") as fa, open("bar", "U") as fb:
    pass
with open("foo", "Ub") as fa, open("bar", "Ub") as fb:
    pass

open("foo", mode="U")
open(name="foo", mode="U")
open(mode="U", name="foo")

with open("foo", mode="U") as f:
    pass
with open(name="foo", mode="U") as f:
    pass
with open(mode="U", name="foo") as f:
    pass

open("foo", mode="Ub")
open(name="foo", mode="Ub")
open(mode="Ub", name="foo")

with open("foo", mode="Ub") as f:
    pass
with open(name="foo", mode="Ub") as f:
    pass
with open(mode="Ub", name="foo") as f:
    pass

open(file="foo", mode='U', buffering=- 1, encoding=None, errors=None, newline=None, closefd=True, opener=None)
open(file="foo", buffering=- 1, encoding=None, errors=None, newline=None, closefd=True, opener=None, mode='U')
open(file="foo", buffering=- 1, encoding=None, errors=None, mode='U', newline=None, closefd=True, opener=None)
open(mode='U', file="foo", buffering=- 1, encoding=None, errors=None, newline=None, closefd=True, opener=None)

open(file="foo", mode='Ub', buffering=- 1, encoding=None, errors=None, newline=None, closefd=True, opener=None)
open(file="foo", buffering=- 1, encoding=None, errors=None, newline=None, closefd=True, opener=None, mode='Ub')
open(file="foo", buffering=- 1, encoding=None, errors=None, mode='Ub', newline=None, closefd=True, opener=None)
open(mode='Ub', file="foo", buffering=- 1, encoding=None, errors=None, newline=None, closefd=True, opener=None)

open = 1
open("foo", "U")
open("foo", "Ur")
open("foo", "Ub")
open("foo", "rUb")
open("foo", "r")
open("foo", "rt")
open("f", "r", encoding="UTF-8")
open("f", "wt")



def foo():
    '''function without params, single line docstring'''
    ''' not a docstring'''
    return


def foo2():
    '''
        function without params, multiline docstring
    '''
    ''' not a docstring'''
    return


def fun_with_params_no_docstring(a, b='''
    not a 
''' '''docstring'''):
    pass

def fun_with_params_no_docstring2(a, b=c[foo():], c=\
    ''' not a docstring '''):
    pass




"""
Should emit:
B020 - on lines 8, 21, and 36
"""

items = [1, 2, 3]

for items in items:
    print(items)

items = [1, 2, 3]

for item in items:
    print(item)

values = {"secret": 123}

for key, value in values.items():
    print(f"{key}, {value}")

for key, values in values.items():
    print(f"{key}, {values}")

# Variables defined in a comprehension are local in scope
# to that comprehension and are therefore allowed.
for var in [var for var in range(10)]:
    print(var)

for var in (var for var in range(10)):
    print(var)

for k, v in {k: v for k, v in zip(range(10), range(10, 20))}.items():
    print(k, v)

# However we still call out reassigning the iterable in the comprehension.
for vars in [i for i in vars]:
    print(vars)

for var in sorted(range(10), key=lambda var: var.real):
    print(var)



def function(
    posonly_nohint,
    posonly_nonboolhint: int,
    posonly_boolhint: bool,
    posonly_boolstrhint: "bool",
    /,
    offset,
    posorkw_nonvalued_nohint,
    posorkw_nonvalued_nonboolhint: int,
    posorkw_nonvalued_boolhint: bool,
    posorkw_nonvalued_boolstrhint: "bool",
    posorkw_boolvalued_nohint=True,
    posorkw_boolvalued_nonboolhint: int = True,
    posorkw_boolvalued_boolhint: bool = True,
    posorkw_boolvalued_boolstrhint: "bool" = True,
    posorkw_nonboolvalued_nohint=1,
    posorkw_nonboolvalued_nonboolhint: int = 2,
    posorkw_nonboolvalued_boolhint: bool = 3,
    posorkw_nonboolvalued_boolstrhint: "bool" = 4,
    *,
    kwonly_nonvalued_nohint,
    kwonly_nonvalued_nonboolhint: int,
    kwonly_nonvalued_boolhint: bool,
    kwonly_nonvalued_boolstrhint: "bool",
    kwonly_boolvalued_nohint=True,
    kwonly_boolvalued_nonboolhint: int = False,
    kwonly_boolvalued_boolhint: bool = True,
    kwonly_boolvalued_boolstrhint: "bool" = True,
    kwonly_nonboolvalued_nohint=5,
    kwonly_nonboolvalued_nonboolhint: int = 1,
    kwonly_nonboolvalued_boolhint: bool = 1,
    kwonly_nonboolvalued_boolstrhint: "bool" = 1,
    **kw,
):
    ...


def used(do):
    return do


used("a", True)
used(do=True)


# Avoid FBT003 for explicitly allowed methods.
"""
FBT003 Boolean positional value on dict
"""
a = {"a": "b"}
a.get("hello", False)
{}.get("hello", False)
{}.setdefault("hello", True)
{}.pop("hello", False)
{}.pop(True, False)
dict.fromkeys(("world",), True)
{}.deploy(True, False)



class SingleLineDocstrings():
    """ Double quotes single line class docstring """
    """ Not a docstring """

    def foo(self, bar="""not a docstring"""):
        """ Double quotes single line method docstring"""
        pass

    class Nested(foo()[:]): """ inline docstring """; pass



"""Test: referencing an import via TypeAlias."""
import sys

import numpy as np

if sys.version_info >= (3, 10):
    from typing import TypeAlias
else:
    from typing_extensions import TypeAlias


CustomInt: TypeAlias = "np.int8 | np.int16"



from contextlib import contextmanager

l = 0
I = 0
O = 0
l: int = 0

a, l = 0, 1
[a, l] = 0, 1
a, *l = 0, 1, 2
a = l = 0

o = 0
i = 0

for l in range(3):
    pass


for a, l in zip(range(3), range(3)):
    pass


def f1():
    global l
    l = 0


def f2():
    l = 0

    def f3():
        nonlocal l
        l = 1

    f3()
    return l


def f4(l, /, I):
    return l, I, O


def f5(l=0, *, I=1):
    return l, I


def f6(*l, **I):
    return l, I


@contextmanager
def ctx1():
    yield 0


with ctx1() as l:
    pass


@contextmanager
def ctx2():
    yield 0, 1


with ctx2() as (a, l):
    pass

try:
    pass
except ValueError as l:
    pass

if (l := 5) > 0:
    pass



from __future__ import annotations

from dataclasses import dataclass
from typing import List, Optional, TypeAlias, Union

from models import (
    Fruit,
    Nut,
)


@dataclass
class Foo:
    x: int
    y: int

    @classmethod
    def a(cls) -> Foo:
        return cls(x=0, y=0)

    @classmethod
    def b(cls) -> "Foo":
        return cls(x=0, y=0)

    @classmethod
    def c(cls) -> Bar:
        return cls(x=0, y=0)

    @classmethod
    def d(cls) -> Fruit:
        return cls(x=0, y=0)


def f(x: int) -> List[int]:
    y = List[int]()
    y.append(x)
    return y


x: Optional[int] = None

MyList: TypeAlias = Union[List[int], List[str]]



import a
import b
x = 1
import os
import sys
def f():
    pass
if True:
    x = 1
    import collections
    import typing
class X: pass
y = 1
import os
import sys
"""Docstring"""

if True:
    import os


def f():
    pass


if True:
    import os
def f():
    pass

if True:
    x = 1
    import collections
    import typing
    class X: pass

if True:
    x = 1
    import collections
    import typing
    def f(): pass



import typing
from typing import Optional
from typing import Union


def f(x: Optional[str]) -> None:
    ...


def f(x: typing.Optional[str]) -> None:
    ...


def f(x: Union[str, int, Union[float, bytes]]) -> None:
    ...


def f(x: typing.Union[str, int]) -> None:
    ...


def f(x: typing.Union[(str, int)]) -> None:
    ...


def f(x: typing.Union[(str, int), float]) -> None:
    ...


def f(x: "Union[str, int, Union[float, bytes]]") -> None:
    ...


def f(x: "typing.Union[str, int]") -> None:
    ...


def f(x: Union["str", int]) -> None:
    ...


def f(x: Union[("str", "int"), float]) -> None:
    ...



"""
Should emit:
B022 - on lines 8
"""

import contextlib
from contextlib import suppress

with contextlib.suppress():
    raise ValueError

with suppress():
    raise ValueError

with contextlib.suppress(ValueError):
    raise ValueError

exceptions_to_suppress = []
if True:
    exceptions_to_suppress.append(ValueError)

with contextlib.suppress(*exceptions_to_suppress):
    raise



def f(tup):
    x, y = tup  # this does NOT trigger F841


def f():
    x, y = 1, 2  # this triggers F841 as it's just a simple assignment where unpacking isn't needed


def f():
    (x, y) = coords = 1, 2  # this does NOT trigger F841
    if x > 1:
        print(coords)


def f():
    (x, y) = coords = 1, 2  # this triggers F841 on coords


def f():
    coords = (x, y) = 1, 2  # this triggers F841 on coords


def f():
    (a, b) = (x, y) = 1, 2  # this triggers F841 on everything



def f():
    "Here's a line without a period"
    ...


def f():
    """Here's a line without a period"""
    ...


def f():
    """
    Here's a line without a period,
    but here's the next line
    """
    ...


def f():
    """Here's a line without a period"""
    ...


def f():
    """
    Here's a line without a period,
    but here's the next line"""
    ...


def f():
    """
    Here's a line without a period,
    but here's the next line with trailing space """
    ...



def f():
    r"Here's a line without a period"
    ...


def f():
    r"""Here's a line without a period"""
    ...


def f():
    r"""
    Here's a line without a period,
    but here's the next line
    """
    ...


def f():
    r"""Here's a line without a period"""
    ...


def f():
    r"""
    Here's a line without a period,
    but here's the next line"""
    ...


def f():
    r"""
    Here's a line without a period,
    but here's the next line with trailing space """
    ...



"""Test: explicit re-export."""

# OK
from .applications import FastAPI as FastAPI

# F401 `background.BackgroundTasks` imported but unused
from .background import BackgroundTasks

# F401 `datastructures.UploadFile` imported but unused
from .datastructures import UploadFile as FileUpload


# OK
import applications as applications

# F401 `background` imported but unused
import background

# F401 `datastructures` imported but unused
import datastructures as structures



d = {}

# OK
safe = "s3cr3t"
password = True
password = safe
password is True
password == 1
d["safe"] = "s3cr3t"

# Errors
password = "s3cr3t"
_pass = "s3cr3t"
passwd = "s3cr3t"
pwd = "s3cr3t"
secret = "s3cr3t"
token = "s3cr3t"
secrete = "s3cr3t"
safe = password = "s3cr3t"
password = safe = "s3cr3t"

d["password"] = "s3cr3t"
d["pass"] = "s3cr3t"
d["passwd"] = "s3cr3t"
d["pwd"] = "s3cr3t"
d["secret"] = "s3cr3t"
d["token"] = "s3cr3t"
d["secrete"] = "s3cr3t"
safe = d["password"] = "s3cr3t"
d["password"] = safe = "s3cr3t"


class MyClass:
    password = "s3cr3t"
    safe = password


MyClass.password = "s3cr3t"
MyClass._pass = "s3cr3t"
MyClass.passwd = "s3cr3t"
MyClass.pwd = "s3cr3t"
MyClass.secret = "s3cr3t"
MyClass.token = "s3cr3t"
MyClass.secrete = "s3cr3t"

password == "s3cr3t"
_pass == "s3cr3t"
passwd == "s3cr3t"
pwd == "s3cr3t"
secret == "s3cr3t"
token == "s3cr3t"
secrete == "s3cr3t"
password == safe == "s3cr3t"



"""
Should emit:
B027 - on lines 12, 15, 18, 22, 30
"""
import abc
from abc import ABC
from abc import abstractmethod
from abc import abstractmethod as notabstract


class AbstractClass(ABC):
    def empty_1(self):  # error
        ...

    def empty_2(self):  # error
        pass

    def empty_3(self):  # error
        """docstring"""
        ...

    def empty_4(self):  # error
        """multiple ellipsis/pass"""
        ...
        pass
        ...
        pass

    @notabstract
    def abstract_0(self):
        ...

    @abstractmethod
    def abstract_1(self):
        ...

    @abstractmethod
    def abstract_2(self):
        pass

    @abc.abstractmethod
    def abstract_3(self):
        ...

    def body_1(self):
        print("foo")
        ...

    def body_2(self):
        self.body_1()


class NonAbstractClass:
    def empty_1(self):  # safe
        ...

    def empty_2(self):  # safe
        pass


# ignore @overload, fixes issue #304
# ignore overload with other imports, fixes #308
import typing
import typing as t
import typing as anything
from typing import Union, overload


class AbstractClass(ABC):
    @overload
    def empty_1(self, foo: str):
        ...

    @typing.overload
    def empty_1(self, foo: int):
        ...

    @t.overload
    def empty_1(self, foo: list):
        ...

    @anything.overload
    def empty_1(self, foo: float):
        ...

    @abstractmethod
    def empty_1(self, foo: Union[str, int, list, float]):
        ...



import math  # not checked

import altair  # unconventional
import matplotlib.pyplot  # unconventional
import numpy  # unconventional
import pandas  # unconventional
import seaborn  # unconventional

import altair as altr  # unconventional
import matplotlib.pyplot as plot  # unconventional
import numpy as np  # unconventional
import pandas as pdas  # unconventional
import seaborn as sbrn  # unconventional

import altair as alt  # conventional
import matplotlib.pyplot as plt  # conventional
import numpy as nmp  # conventional
import pandas as pd  # conventional
import seaborn as sns  # conventional



from abc import ABCMeta

import pydantic


class Class:
    def bad_method(this):
        pass

    if False:

        def extra_bad_method(this):
            pass

    def good_method(self):
        pass

    @classmethod
    def class_method(cls):
        pass

    @staticmethod
    def static_method(x):
        return x

    @pydantic.validator
    def lower(cls, my_field: str) -> str:
        pass

    @pydantic.validator("my_field")
    def lower(cls, my_field: str) -> str:
        pass

    def __init__(self):
        ...

    def __new__(cls, *args, **kwargs):
        ...

    def __init_subclass__(self, default_name, **kwargs):
        ...


class MetaClass(ABCMeta):
    def bad_method(self):
        pass

    def good_method(cls):
        pass


def func(x):
    return x



def f() -> None:
    # Valid
    a = 1  # noqa

    # Valid
    b = 2  # noqa: F841

    # Invalid
    c = 1  # noqa
    print(c)

    # Invalid
    d = 1  # noqa: E501

    # Invalid
    d = 1  # noqa: F841, E501

    # Invalid (and unimplemented)
    d = 1  # noqa: F841, W191

    # Invalid (but external)
    d = 1  # noqa: F841, V101

    # fmt: off
    # Invalid - no space before #
    d = 1# noqa: E501

    # Invalid - many spaces before #
    d = 1                       # noqa: E501
    # fmt: on


# Valid
_ = """Lorem ipsum dolor sit amet.

    https://github.com/PyCQA/pycodestyle/pull/258/files#diff-841c622497a8033d10152bfdfb15b20b92437ecdea21a260944ea86b77b51533

Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
"""  # noqa: E501

# Valid
_ = """Lorem ipsum dolor sit amet.

    https://github.com/PyCQA/pycodestyle/pull/258/files#diff-841c622497a8033d10152bfdfb15b20b92437ecdea21a260944ea86b77b51533

Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
"""  # noqa

# Invalid
_ = """Lorem ipsum dolor sit amet.

    https://github.com/PyCQA/pycodestyle/pull/258/files#diff-841c622497a8033d10152bfdfb15b20b92437ecdea21a260944ea86b77b51533

Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
"""  # noqa: E501, F841

# Invalid
_ = """Lorem ipsum dolor sit amet.

    https://github.com/PyCQA/pycodestyle/pull/258/files#diff-841c622497a8033d10152bfdfb15b20b92437ecdea21a260944ea86b77b51533

Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor.
"""  # noqa: E501

# Invalid
_ = """Lorem ipsum dolor sit amet.

    https://github.com/PyCQA/pycodestyle/pull/258/files#diff-841c622497a8033d10152bfdfb15b20b92437ecdea21a260944ea86b77b51533

Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor.
"""  # noqa



"""Check for else branches on loops with break and return only."""


def test_return_for():
    """else + return is not acceptable."""
    for i in range(10):
        if i % 2:
            return i
    else:  # [useless-else-on-loop]
        print("math is broken")
    return None


def test_return_while():
    """else + return is not acceptable."""
    while True:
        return 1
    else:  # [useless-else-on-loop]
        print("math is broken")
    return None


while True:

    def short_fun():
        """A function with a loop."""
        for _ in range(10):
            break

else:  # [useless-else-on-loop]
    print("or else!")


while True:
    while False:
        break
else:  # [useless-else-on-loop]
    print("or else!")

for j in range(10):
    pass
else:  # [useless-else-on-loop]
    print("fat chance")
    for j in range(10):
        break


def test_return_for2():
    """no false positive for break in else

    https://bitbucket.org/logilab/pylint/issue/117/useless-else-on-loop-false-positives
    """
    for i in range(10):
        for _ in range(i):
            if i % 2:
                break
        else:
            break
    else:
        print("great math")


def test_break_in_orelse_deep():
    """no false positive for break in else deeply nested"""
    for _ in range(10):
        if 1 < 2:  # pylint: disable=comparison-of-constants
            for _ in range(3):
                if 3 < 2:  # pylint: disable=comparison-of-constants
                    break
            else:
                break
    else:
        return True
    return False


def test_break_in_orelse_deep2():
    """should rise a useless-else-on-loop message, as the break statement is only
    for the inner for loop
    """
    for _ in range(10):
        if 1 < 2:  # pylint: disable=comparison-of-constants
            for _ in range(3):
                if 3 < 2:  # pylint: disable=comparison-of-constants
                    break
            else:
                print("all right")
    else:  # [useless-else-on-loop]
        return True
    return False


def test_break_in_orelse_deep3():
    """no false positive for break deeply nested in else"""
    for _ in range(10):
        for _ in range(3):
            pass
        else:
            if 1 < 2:  # pylint: disable=comparison-of-constants
                break
    else:
        return True
    return False



#: E731
f = lambda x: 2 * x
#: E731
f = lambda x: 2 * x
#: E731
while False:
    this = lambda y, z: 2 * x


f = object()
f.method = lambda: "Method"

f = {}
f["a"] = lambda x: x ** 2

f = []
f.append(lambda x: x ** 2)

f = g = lambda x: x ** 2

lambda: "no-op"



# ASCII literals should be replaced by a bytes literal
"foo".encode("utf-8")  # b"foo"
"foo".encode("u8")  # b"foo"
"foo".encode()  # b"foo"
"foo".encode("UTF8")  # b"foo"
U"foo".encode("utf-8")  # b"foo"
"foo".encode(encoding="utf-8")  # b"foo"
"""
Lorem

Ipsum
""".encode(
    "utf-8"
)
# b"""
# Lorem
#
# Ipsum
# """

# `encode` on variables should not be processed.
string = "hello there"
string.encode("utf-8")

bar = "bar"
f"foo{bar}".encode("utf-8")  # f"foo{bar}".encode()
encoding = "latin"
"foo".encode(encoding)
f"foo{bar}".encode(encoding)

# `encode` with custom args and kwargs should not be processed.
"foo".encode("utf-8", errors="replace")
"foo".encode("utf-8", "replace")
"foo".encode(errors="replace")
"foo".encode(encoding="utf-8", errors="replace")

# `encode` with custom args and kwargs on unicode should not be processed.
"unicode text©".encode("utf-8", errors="replace")
"unicode text©".encode("utf-8", "replace")
"unicode text©".encode(errors="replace")
"unicode text©".encode(encoding="utf-8", errors="replace")

# Unicode literals should only be stripped of default encoding.
"unicode text©".encode("utf-8")  # "unicode text©".encode()
"unicode text©".encode()
"unicode text©".encode(encoding="UTF8")  # "unicode text©".encode()

r"fo\o".encode("utf-8")  # br"fo\o"
u"foo".encode("utf-8")  # b"foo"
R"fo\o".encode("utf-8")  # br"fo\o"
U"foo".encode("utf-8")  # b"foo"
print("foo".encode())  # print(b"foo")



"""
Should emit:
B024 - on lines 17, 34, 52, 58, 69, 74, 79, 84, 89
"""

import abc
import abc as notabc
from abc import ABC, ABCMeta
from abc import abstractmethod
from abc import abstractmethod as abstract
from abc import abstractmethod as abstractaoeuaoeuaoeu
from abc import abstractmethod as notabstract

import foo


class Base_1(ABC):  # error
    def method(self):
        foo()


class Base_2(ABC):
    @abstractmethod
    def method(self):
        foo()


class Base_3(ABC):
    @abc.abstractmethod
    def method(self):
        foo()


class Base_4(ABC):
    @notabc.abstractmethod
    def method(self):
        foo()


class Base_5(ABC):
    @abstract
    def method(self):
        foo()


class Base_6(ABC):
    @abstractaoeuaoeuaoeu
    def method(self):
        foo()


class Base_7(ABC):  # error
    @notabstract
    def method(self):
        foo()


class MetaBase_1(metaclass=ABCMeta):  # error
    def method(self):
        foo()


class MetaBase_2(metaclass=ABCMeta):
    @abstractmethod
    def method(self):
        foo()


class abc_Base_1(abc.ABC):  # error
    def method(self):
        foo()


class abc_Base_2(metaclass=abc.ABCMeta):  # error
    def method(self):
        foo()


class notabc_Base_1(notabc.ABC):  # error
    def method(self):
        foo()


class multi_super_1(notabc.ABC, abc.ABCMeta):  # safe
    def method(self):
        foo()


class multi_super_2(notabc.ABC, metaclass=abc.ABCMeta):  # safe
    def method(self):
        foo()


class non_keyword_abcmeta_1(ABCMeta):  # safe
    def method(self):
        foo()


class non_keyword_abcmeta_2(abc.ABCMeta):  # safe
    def method(self):
        foo()


# very invalid code, but that's up to mypy et al to check
class keyword_abc_1(metaclass=ABC):  # safe
    def method(self):
        foo()


class keyword_abc_2(metaclass=abc.ABC):  # safe
    def method(self):
        foo()


class abc_set_class_variable_1(ABC):  # safe
    foo: int


class abc_set_class_variable_2(ABC):  # safe
    foo = 2


class abc_set_class_variable_3(ABC):  # safe
    foo: int = 2


# this doesn't actually declare a class variable, it's just an expression
class abc_set_class_variable_4(ABC):  # error
    foo



"""Lorem ipsum dolor sit amet.

    https://github.com/PyCQA/pycodestyle/pull/258/files#diff-841c622497a8033d10152bfdfb15b20b92437ecdea21a260944ea86b77b51533

Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
"""

_ = """Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.

    https://github.com/PyCQA/pycodestyle/pull/258/files#diff-841c622497a8033d10152bfdfb15b20b92437ecdea21a260944ea86b77b51533

Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
"""  # noqa: E501

_ = "---------------------------------------------------------------------------AAAAAAA"
_ = "---------------------------------------------------------------------------亜亜亜亜亜亜亜"


def caller(string: str) -> None:
    assert string is not None


caller(
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
    """,
)


caller(
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
    """,  # noqa: E501
)

multiple_strings_per_line = {
    # OK
    "Lorem ipsum dolor": "sit amet",
    # E501 Line too long
    "Lorem ipsum dolor": "sit amet consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.",
    # E501 Line too long
    "Lorem ipsum dolor": """
sit amet  consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
""",
    # OK
    "Lorem ipsum dolor": "sit amet consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.",  # noqa: E501
    # OK
    "Lorem ipsum dolor": """
sit amet  consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
""",  # noqa: E501
}

# OK
# A very long URL: https://loooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooong.url.com

# OK
# https://loooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooong.url.com



import io
# Old MacDonald had a farm,
# EIEIO
# And on his farm he had a cow,
# EIEIO
# With a moo-moo here and a moo-moo there
# Here a moo, there a moo, everywhere moo-moo
# Old MacDonald had a farm,
# EIEIO
from errno import EIO
import abc



"""
Should emit:
B026 - on lines 16, 17, 18, 19, 20, 21
"""


def foo(bar, baz, bam):
    print(bar, baz, bam)


bar_baz = ["bar", "baz"]

foo("bar", "baz", bam="bam")
foo("bar", baz="baz", bam="bam")
foo(bar="bar", baz="baz", bam="bam")
foo(bam="bam", *["bar", "baz"])
foo(bam="bam", *bar_baz)
foo(baz="baz", bam="bam", *["bar"])
foo(bar="bar", baz="baz", bam="bam", *[])
foo(bam="bam", *["bar"], *["baz"])
foo(*["bar"], bam="bam", *["baz"])



for i in range(10):
    continue
else:
    continue

i = 0
while i < 10:
    i += 1
    continue


def f():
    for i in range(10):
        continue

    continue


class Foo:
    continue


continue



# pylint: disable=missing-docstring,unused-variable
import asyncio

async def nested():
    return 42

async def main():
    nested()
    print(await nested())  # This is okay

def not_async():
    print(await nested())  # [await-outside-async]


async def func(i):
    return i**2

async def okay_function():
    var = [await func(i) for i in range(5)]  # This should be okay


# Test nested functions
async def func2():
    def inner_func():
        await asyncio.sleep(1)  # [await-outside-async]


def outer_func():
    async def inner_func():
        await asyncio.sleep(1)



from . import sibling
from .sibling import example

from .. import parent
from ..parent import example

from ... import grandparent
from ...grandparent import example

import other
import other.example
from other import example



import sys

from setuptools import setup

sys.stderr.write(
    """
===============================
Unsupported installation method
===============================
ruff no longer supports installation with `python setup.py install`.
Please use `python -m pip install .` instead.
"""
)
sys.exit(1)


# The below code will never execute, however GitHub is particularly
# picky about where it finds Python packaging metadata.
# See: https://github.com/github/feedback/discussions/6456
#
# To be removed once GitHub catches up.

setup(name="ruff", install_requires=[])



try:
    pass
except ValueError:
    pass
except Exception as e:
    raise e
finally:
    pass


try:
    pass
except BaseException as e:
    raise e
except TypeError:
    pass
else:
    pass


try:
    pass
except Exception as e:
    raise e
except BaseException:
    pass


try:
    pass
except Exception:
    pass
finally:
    try:
        pass
    except BaseException as e:
        raise e


try:
    pass
except Exception as e:
    try:
        raise e
    except BaseException:
        pass


try:
    try:
        pass
    except BaseException as e:
        raise e
except Exception:
    pass


try:
    pass
except Exception as e:
    raise bad
except BaseException:
    pass



###
# Errors
###
def x():
    a = 1
    return a  # error


def x():
    b, a = 1, 2
    print(b)
    return a  # error


def x():
    a = 1
    print()
    return a  # error


def x():
    a = 1
    print(a)
    a = 2
    return a  # error


def x():
    a = 1
    if True:
        return a  # error
    a = 2
    print(a)
    return a


# Can be refactored false positives
# https://github.com/afonasev/flake8-return/issues/47#issuecomment-1122571066
def get_bar_if_exists(obj):
    result = ""
    if hasattr(obj, "bar"):
        result = str(obj.bar)
    return result


# https://github.com/afonasev/flake8-return/issues/47#issue-641117366
def x():
    formatted = _USER_AGENT_FORMATTER.format(format_string, **values)
    # clean up after any blank components
    formatted = formatted.replace("()", "").replace("  ", " ").strip()
    return formatted


# https://github.com/afonasev/flake8-return/issues/47#issue-641117366
def user_agent_username(username=None):

    if not username:
        return ""

    username = username.replace(" ", "_")  # Avoid spaces or %20.
    try:
        username.encode("ascii")  # just test, but not actually use it
    except UnicodeEncodeError:
        username = quote(username.encode("utf-8"))
    else:
        # % is legal in the default $wgLegalTitleChars
        # This is so that ops know the real pywikibot will not
        # allow a useragent in the username to allow through a hand-coded
        # percent-encoded value.
        if "%" in username:
            username = quote(username)
    return username


###
# Non-errors
###
def x(y):
    a = 1
    print(a)
    return a


def x():
    a = 1
    if y:
        return a
    a = a + 2
    print(a)
    return a


def x():
    a = {}
    a["b"] = 2
    return a


def x():
    a = []
    a.append(2)
    return a


def x():
    a = lambda x: x
    a()
    return a


# ignore unpacking
def x():
    b, a = [1, 2]
    return a


def x():
    val = ""
    for i in range(5):
        val = val + str(i)
    return val


def x():
    val = ""
    i = 5
    while i:
        val = val + str(i)
        i = i - x
    return val


# Test cases for using value for assignment then returning it
# See:https://github.com/afonasev/flake8-return/issues/47
def resolve_from_url(self, url: str) -> dict:
    local_match = self.local_scope_re.match(url)
    if local_match:
        schema = get_schema(name=local_match.group(1))
        self.store[url] = schema
        return schema
    raise NotImplementedError(...)


my_dict = {}


def my_func():
    foo = calculate_foo()
    my_dict["foo_result"] = foo
    return foo


# Refactored incorrect false positives
# See test cases above marked: "Can be refactored false positives"
# https://github.com/afonasev/flake8-return/issues/47#issuecomment-1122571066
def get_bar_if_exists(obj):
    if hasattr(obj, "bar"):
        return str(obj.bar)
    return ""


# https://github.com/afonasev/flake8-return/issues/47#issue-641117366
def x():
    formatted = _USER_AGENT_FORMATTER.format(format_string, **values)
    # clean up after any blank components
    return formatted.replace("()", "").replace("  ", " ").strip()


# https://github.com/afonasev/flake8-return/issues/47#issue-641117366
def user_agent_username(username=None):

    if not username:
        return ""

    username = username.replace(" ", "_")  # Avoid spaces or %20.
    try:
        username.encode("ascii")  # just test,
        # but not actually use it
    except UnicodeEncodeError:
        username = quote(username.encode("utf-8"))
    else:
        # % is legal in the default $wgLegalTitleChars
        # This is so that ops know the real pywikibot will not
        # allow a useragent in the username to allow through a
        # hand-coded percent-encoded value.
        if "%" in username:
            username = quote(username)
    return username


# https://github.com/afonasev/flake8-return/issues/116#issue-1367575481
def no_exception_loop():
    success = False
    for _ in range(10):
        try:
            success = True
        except Exception:
            print("exception")
    return success


# https://github.com/afonasev/flake8-return/issues/116#issue-1367575481
def no_exception():
    success = False
    try:
        success = True
    except Exception:
        print("exception")
    return success


# https://github.com/afonasev/flake8-return/issues/116#issue-1367575481
def exception():
    success = True
    try:
        print("raising")
        raise Exception
    except Exception:
        success = False
    return success


# https://github.com/afonasev/flake8-return/issues/66
def close(self):
    any_failed = False
    for task in self.tasks:
        try:
            task()
        except BaseException:
            any_failed = True
            report(traceback.format_exc())
    return any_failed



###
# Errors
###
def foo2(x, y, w, z):
    if x:  # [no-else-raise]
        a = 1
        raise Exception(y)
    elif z:
        b = 2
        raise Exception(w)
    else:
        c = 3
        raise Exception(z)


def foo5(x, y, z):
    if x:  # [no-else-raise]
        if y:
            a = 4
        else:
            b = 2
        raise Exception(x)
    elif z:
        raise Exception(y)
    else:
        c = 3
    raise Exception(z)


def foo1(x, y, z):
    if x:  # [no-else-raise]
        a = 1
        raise Exception(y)
    else:
        b = 2
        raise Exception(z)


def foo3(x, y, z):
    if x:
        a = 1
        if y:  # [no-else-raise]
            b = 2
            raise Exception(y)
        else:
            c = 3
            raise Exception(x)
    else:
        d = 4
        raise Exception(z)


def foo4(x, y):
    if x:  # [no-else-raise]
        if y:
            a = 4
        else:
            b = 2
        raise Exception(x)
    else:
        c = 3
    raise Exception(y)


def foo6(x, y):
    if x:
        if y:  # [no-else-raise]
            a = 4
            raise Exception(x)
        else:
            b = 2
    else:
        c = 3
    raise Exception(y)


def bar4(x):
    if x:  # [no-else-raise]
        raise Exception(True)
    else:
        try:
            raise Exception(False)
        except ValueError:
            raise Exception(None)


###
# Non-errors
###
def bar1(x, y, z):
    if x:
        raise Exception(y)
    raise Exception(z)


def bar2(w, x, y, z):
    if x:
        raise Exception(y)
    if z:
        a = 1
    else:
        raise Exception(w)
    raise Exception(None)


def bar3(x, y, z):
    if x:
        if z:
            raise Exception(y)
    else:
        raise Exception(z)
    raise Exception(None)



class Foo1:
    """abc"""


class Foo2:
    """abc"""

    a = 2
    "str"  # Str (no raise)
    f"{int}"  # JoinedStr (no raise)
    1j  # Number (complex)
    1  # Number (int)
    1.0  # Number (float)
    b"foo"  # Binary
    True  # NameConstant (True)
    False  # NameConstant (False)
    None  # NameConstant (None)
    [1, 2]  # list
    {1, 2}  # set
    {"foo": "bar"}  # dict


class Foo3:
    123
    a = 2
    "str"
    1


def foo1():
    """my docstring"""


def foo2():
    """my docstring"""
    a = 2
    "str"  # Str (no raise)
    f"{int}"  # JoinedStr (no raise)
    1j  # Number (complex)
    1  # Number (int)
    1.0  # Number (float)
    b"foo"  # Binary
    True  # NameConstant (True)
    False  # NameConstant (False)
    None  # NameConstant (None)
    [1, 2]  # list
    {1, 2}  # set
    {"foo": "bar"}  # dict


def foo3():
    123
    a = 2
    "str"
    3


def foo4():
    ...



import unittest


class Suite(unittest.TestCase):
    def test(self) -> None:
        self.assertEquals (1, 2)
        self.assertEquals(1, 2)
        self.assertEqual(3, 4)
        self.failUnlessAlmostEqual(1, 1.1)
        self.assertNotRegexpMatches("a", "b")



'''
Single quotes multiline module docstring
'''

'''
this is not a docstring
'''

l = []

class Cls(MakeKlass('''
    class params \t not a docstring
''')):
    '''
    Single quotes multiline class docstring
    '''

    '''
    this is not a docstring
    '''

    # The colon in the list indexing below is an edge case for the docstring scanner
    def f(self, bar='''
        definitely not a docstring''',
        val=l[Cls():3]):
        '''
        Single quotes multiline function docstring
        '''

        some_expression = 'hello world'

        '''
        this is not a docstring
        '''

        if l:
            '''
            Looks like a docstring, but in reality it isn't - only modules, classes and functions
            '''
            pass



"""
Double quotes multiline module docstring
"""

"""
this is not a docstring
"""

l = []

class Cls:
    """
    Double quotes multiline class docstring
    """

    """
    this is not a docstring
    """

    # The colon in the list indexing below is an edge case for the docstring scanner
    def f(self, bar="""
        definitely not a docstring""",
        val=l[Cls():3]):
        """
        Double quotes multiline function docstring
        """

        some_expression = 'hello world'

        """
        this is not a docstring
        """

        if l:
            """
            Looks like a docstring, but in reality it isn't - only modules, classes and functions
            """
            pass



import sys
from sys import version_info

print("{}.{}".format(*sys.version_info))
PY3 = sys.version_info[0] >= 3

PY3 = sys.version_info[0] == 3
PY3 = version_info[0] == 3
PY2 = sys.version_info[0] != 3
PY2 = version_info[0] != 3



# Complexity = 1
def trivial():
    pass


# Complexity = 1
def expr_as_statement():
    0xF00D


# Complexity = 1
def sequential(n):
    k = n + 4
    s = k + n
    return s


# Complexity = 3
def if_elif_else_dead_path(n):
    if n > 3:
        return "bigger than three"
    elif n > 4:
        return "is never executed"
    else:
        return "smaller than or equal to three"


# Complexity = 3
def nested_ifs():
    if n > 3:
        if n > 4:
            return "bigger than four"
        else:
            return "bigger than three"
    else:
        return "smaller than or equal to three"


# Complexity = 2
def for_loop():
    for i in range(10):
        print(i)


# Complexity = 2
def for_else(mylist):
    for i in mylist:
        print(i)
    else:
        print(None)


# Complexity = 2
def recursive(n):
    if n > 4:
        return f(n - 1)
    else:
        return n


# Complexity = 3
def nested_functions():
    def a():
        def b():
            pass

        b()

    a()


# Complexity = 4
def try_else():
    try:
        print(1)
    except TypeA:
        print(2)
    except TypeB:
        print(3)
    else:
        print(4)


# Complexity = 3
def nested_try_finally():
    try:
        try:
            print(1)
        finally:
            print(2)
    finally:
        print(3)


# Complexity = 3
async def foobar(a, b, c):
    await whatever(a, b, c)
    if await b:
        pass
    async with c:
        pass
    async for x in a:
        pass


# Complexity = 1
def annotated_assign():
    x: Any = None


# Complexity = 9
class Class:
    def handle(self, *args, **options):
        if args:
            return

        class ServiceProvider:
            def a(self):
                pass

            def b(self, data):
                if not args:
                    pass

        class Logger:
            def c(*args, **kwargs):
                pass

            def error(self, message):
                pass

            def info(self, message):
                pass

            def exception(self):
                pass

        return ServiceProvider(Logger())



"""
Should emit:
B904 - on lines 10, 11 and 16
"""

try:
    raise ValueError
except ValueError:
    if "abc":
        raise TypeError
    raise UserWarning
except AssertionError:
    raise  # Bare `raise` should not be an error
except Exception as err:
    assert err
    raise Exception("No cause here...")
except BaseException as base_err:
    # Might use this instead of bare raise with the `.with_traceback()` method
    raise base_err
finally:
    raise Exception("Nothing to chain from, so no warning here")

try:
    raise ValueError
except ValueError:
    # should not emit, since we are not raising something
    def proxy():
        raise NameError


try:
    from preferred_library import Thing
except ImportError:
    try:
        from fallback_library import Thing
    except ImportError:

        class Thing:
            def __getattr__(self, name):
                # same as the case above, should not emit.
                raise AttributeError


try:
    from preferred_library import Thing
except ImportError:
    try:
        from fallback_library import Thing
    except ImportError:

        def context_switch():
            try:
                raise ValueError
            except ValueError:
                raise
